@@ -4,10 +4,12 @@
 # Copyright 2019 Brian Pepple
 
 import io
+import logging
 import os
-import sys
 import tempfile
 import zipfile
+from pathlib import Path
+from typing import List, Optional, Text
 
 from natsort import natsorted
 from PIL import Image
@@ -16,17 +18,17 @@ from .comicinfoxml import ComicInfoXml
 from .filenameparser import FileNameParser
 from .genericmetadata import GenericMetadata
 
-sys.path.insert(0, os.path.abspath("."))
+logger = logging.getLogger(__name__)
 
 
 class ZipArchiver:
 
     """ZIP implementation"""
 
-    def __init__(self, path):
+    def __init__(self, path: Path) -> None:
         self.path = path
 
-    def read_archive_file(self, archive_file):
+    def read_archive_file(self, archive_file: str) -> bytes:
         """Read the contents of a comic archive"""
 
         data = ""
@@ -35,24 +37,22 @@ class ZipArchiver:
         try:
             data = zip_file.read(archive_file)
         except zipfile.BadZipfile as bad_zip_error:
-            print(
-                f"bad zipfile [{bad_zip_error}]: {self.path} :: {archive_file}",
-                file=sys.stderr,
+            logger.exception(
+                f"bad zipfile [{bad_zip_error}]: {self.path} :: {archive_file}"
             )
             zip_file.close()
             raise IOError
         except Exception as exception_error:
-            zip_file.close()
-            print(
-                f"bad zipfile [{exception_error}]: {self.path} :: {archive_file}",
-                file=sys.stderr,
+            logger.exception(
+                f"bad zipfile [{exception_error}]: {self.path} :: {archive_file}"
             )
+            zip_file.close()
             raise IOError
         finally:
             zip_file.close()
         return data
 
-    def remove_archive_file(self, archive_file):
+    def remove_archive_file(self, archive_file: str) -> bool:
         """Returns a boolean when attempting to remove a file from an archive"""
 
         try:
@@ -62,7 +62,7 @@ class ZipArchiver:
         else:
             return True
 
-    def write_archive_file(self, archive_file, data):
+    def write_archive_file(self, archive_file: str, data: str) -> bool:
         #  At the moment, no other option but to rebuild the whole
         #  zip archive w/o the indicated file. Very sucky, but maybe
         # another solution can be found
@@ -77,10 +77,10 @@ class ZipArchiver:
             zip_file.close()
             return True
         except (zipfile.BadZipfile, zipfile.LargeZipFile) as exception_error:
-            print(f"Error writing zipfile: {exception_error}.")
+            logger.exception(f"Error writing zipfile: {exception_error}.")
             return False
 
-    def get_archive_filename_list(self):
+    def get_archive_filename_list(self) -> List[Text]:
         """Returns a list of the filenames in an archive"""
 
         try:
@@ -89,23 +89,18 @@ class ZipArchiver:
             zip_file.close()
             return namelist
         except Exception as exception_error:
-            print(
-                f"Unable to get zipfile list [{exception_error}]: {self.path}",
-                file=sys.stderr,
+            logger.exception(
+                f"Unable to get zipfile list [{exception_error}]: {self.path}"
             )
             return []
 
-    def rebuild_zipfile(self, exclude_list):
+    def rebuild_zipfile(self, exclude_list: List[str]) -> None:
         """Zip helper func
 
         This recompresses the zip archive, without the files in the exclude_list
         """
-
-        # print ">> sys.stderr, Rebuilding zip {0} without {1}".format(
-        #                                            self.path, exclude_list )
-
         # generate temp file
-        tmp_fd, tmp_name = tempfile.mkstemp(dir=os.path.dirname(self.path))
+        tmp_fd, tmp_name = tempfile.mkstemp(dir=self.path.parent)
         os.close(tmp_fd)
 
         zin = zipfile.ZipFile(self.path, "r")
@@ -119,37 +114,10 @@ class ZipArchiver:
         zin.close()
 
         # replace with the new file
-        os.remove(self.path)
+        self.path.unlink()
         os.rename(tmp_name, self.path)
 
 
-# ------------------------------------------
-
-
-class UnknownArchiver:
-
-    """Unknown implementation"""
-
-    def __init__(self, path):
-        self.path = path
-
-    @classmethod
-    def read_archive_file(cls):
-        return ""
-
-    @classmethod
-    def write_archive_file(cls, archive_file, data):
-        return False
-
-    @classmethod
-    def remove_archive_file(cls, archive_file):
-        return False
-
-    @classmethod
-    def get_archive_filename_list(cls):
-        return []
-
-
 # ------------------------------------------------------------------
 
 
@@ -162,40 +130,36 @@ class ComicArchive:
 
         Zip, Unknown = list(range(2))
 
-    def __init__(self, path):
+    def __init__(self, path: Path) -> None:
         self.path = path
 
         self.ci_xml_filename = "ComicInfo.xml"
-        self.has_md = None
-        self.page_count = None
-        self.page_list = None
-        self.metadata = None
+        self.has_md: Optional[bool] = None
+        self.page_count: Optional[int] = None
+        self.page_list: Optional[List[str]] = None
+        self.metadata: Optional[GenericMetadata] = None
 
-        self.archive_type = self.ArchiveType.Unknown
-        self.archiver = UnknownArchiver(self.path)
+        self.archive_type: int = self.ArchiveType.Zip
+        self.archiver = ZipArchiver(self.path)
 
-        if self.zip_test():
-            self.archive_type = self.ArchiveType.Zip
-            self.archiver = ZipArchiver(self.path)
-
-    def reset_cache(self):
+    def reset_cache(self) -> None:
         """Clears the cached data"""
         self.has_md = None
         self.page_count = None
         self.page_list = None
         self.metadata = None
 
-    def zip_test(self):
+    def zip_test(self) -> bool:
         """Tests whether an archive is a zipfile"""
 
         return zipfile.is_zipfile(self.path)
 
-    def is_zip(self):
+    def is_zip(self) -> bool:
         """Returns a boolean as to whether an archive is a zipfile"""
 
         return self.archive_type == self.ArchiveType.Zip
 
-    def is_writable(self):
+    def is_writable(self) -> bool:
         """Returns a boolean as to whether an archive is writable"""
 
         if self.archive_type == self.ArchiveType.Unknown:
@@ -203,12 +167,12 @@ class ComicArchive:
 
         return bool(os.access(self.path, os.W_OK))
 
-    def seems_to_be_a_comic_archive(self):
+    def seems_to_be_a_comic_archive(self) -> bool:
         """Returns a boolean as to whether the file is a comic archive"""
 
         return bool((self.is_zip()) and (self.get_number_of_pages() > 0))
 
-    def get_page(self, index):
+    def get_page(self, index: int) -> Optional[bytes]:
         """Returns an image(page) from an archive"""
 
         image_data = None
@@ -219,11 +183,11 @@ class ComicArchive:
             try:
                 image_data = self.archiver.read_archive_file(filename)
             except IOError:
-                print("Error reading in page.", file=sys.stderr)
+                logger.exception("Error reading in page.")
 
         return image_data
 
-    def get_page_name(self, index):
+    def get_page_name(self, index: int) -> Optional[str]:
         """Returns the page name from an index"""
 
         if index is None:
@@ -237,7 +201,7 @@ class ComicArchive:
 
         return page_list[index]
 
-    def get_page_name_list(self, sort_list=True):
+    def get_page_name_list(self, sort_list: bool = True) -> List[str]:
         """Returns a list of page names from an archive"""
 
         if self.page_list is None:
@@ -260,22 +224,23 @@ class ComicArchive:
             # make a sub-list of image files
             self.page_list = []
             for name in files:
+                name_path = Path(name)
                 if (
-                    name[-4:].lower() in [".jpg", "jpeg", ".png", ".gif", "webp"]
-                    and os.path.basename(name)[0] != "."
+                    name_path.suffix.lower() in [".jpg", "jpeg", ".png", ".gif", "webp"]
+                    and name_path.name[0] != "."
                 ):
                     self.page_list.append(name)
 
         return self.page_list
 
-    def get_number_of_pages(self):
+    def get_number_of_pages(self) -> int:
         """Returns the number of pages in an archive"""
 
         if self.page_count is None:
             self.page_count = len(self.get_page_name_list())
         return self.page_count
 
-    def read_metadata(self):
+    def read_metadata(self) -> GenericMetadata:
         """Reads the metadata from an archive if present"""
 
         if self.metadata is None:
@@ -299,17 +264,19 @@ class ComicArchive:
 
         return self.metadata
 
-    def read_raw_metadata(self):
+    def read_raw_metadata(self) -> Optional[str]:
         if not self.has_metadata():
             return None
         try:
-            raw_metadata = self.archiver.read_archive_file(self.ci_xml_filename)
+            tmp_raw_metadata = self.archiver.read_archive_file(self.ci_xml_filename)
+            # Convert bytes to str. Is it safe to decode with utf-8?
+            raw_metadata = tmp_raw_metadata.decode("utf-8")
         except IOError:
             print("Error reading in raw CIX!")
-            raw_metadata = ""
+            raw_metadata = None
         return raw_metadata
 
-    def write_metadata(self, metadata):
+    def write_metadata(self, metadata: GenericMetadata) -> bool:
         """Write the metadata to the archive"""
 
         if metadata is None:
@@ -325,7 +292,7 @@ class ComicArchive:
         self.reset_cache()
         return write_success
 
-    def remove_metadata(self):
+    def remove_metadata(self) -> bool:
         """Remove the metadata from the archive if present"""
 
         if self.has_metadata():
@@ -337,7 +304,7 @@ class ComicArchive:
             return write_success
         return True
 
-    def has_metadata(self):
+    def has_metadata(self) -> bool:
         """Checks to see if the archive has metadata"""
 
         if self.has_md is None:
@@ -353,7 +320,9 @@ class ComicArchive:
                 self.has_md = True
         return self.has_md
 
-    def apply_archive_info_to_metadata(self, metadata, calc_page_sizes=False):
+    def apply_archive_info_to_metadata(
+        self, metadata: GenericMetadata, calc_page_sizes: bool = False
+    ) -> None:
         """Apply page information from the archive to the metadata"""
 
         metadata.page_count = self.get_number_of_pages()
@@ -378,7 +347,7 @@ class ComicArchive:
                         except IOError:
                             page["ImageSize"] = str(len(data))
 
-    def metadata_from_filename(self, parse_scan_info=True):
+    def metadata_from_filename(self, parse_scan_info: bool = True) -> GenericMetadata:
         """Attempts to get the metadata from the filename"""
 
         metadata = GenericMetadata()
@@ -395,10 +364,10 @@ class ComicArchive:
         if fnp.year != "":
             metadata.year = fnp.year
         if fnp.issue_count != "":
-            metadata.issueCount = fnp.issue_count
+            metadata.issue_count = fnp.issue_count
         if parse_scan_info and fnp.remainder != "":
-            metadata.scanInfo = fnp.remainder
+            metadata.scan_info = fnp.remainder
 
-        metadata.isEmpty = False
+        metadata.is_empty = False
 
         return metadata
